# BEGIN COPYRIGHT
# *************************************************************************
# Licensed Materials - Property of IBM 
# 5724-L01, 5655-N53, 5724-I82, 5655-R15
# Copyright IBM Corporation 2012. All rights reserved. 
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
# *************************************************************************
# END COPYRIGHT
#
#**************************************************************************

# change the SOAP connection timeout because undeploying takes a long time.
# 
r'''
Usage:
<BPM-INSTALL-HOME>\profiles\<PROFILE-NAME>\bin>wsadmin -connType SOAP -port <APP_CLUSTER_SOAP_PORT> -lang jython -user <user-name> -password <password> 
                                            -f <BPM-INSTALL-HOME>\BPM\Lombardi\tools\cleanup\BPMundeploy-archive.py [-age <ndays>] [-archive] [-paAcronym <pa acronym>]
'''
# TODO: Change the "AGE" crieria for creation of snapshots, by default it includes all (snapshots created more than "AGE" days ago)
# The com.ibm.SOAP.requestTimeout setting can be changed in <WAS_HOME>\profiles\<profile_name>\properties\soap.client.props
import os, sys, calendar, time, random, string
r = random
s = string

import sys
import os.path
import time
from java.lang import System

True = 1
False = 0

# If a snapshot was created more than or equal to AGE days ago, 
# it becomes a candidate for undeployment 
AGE = 3
# By default don't archive the undeployed snapshots. (change this by using the -archive flag)
DONT_ARCHIVE = True
# The user can specify a single PA to be the target of this script using the -paAcronym flag
PA_TARGET = ''

# From more logging set debug to 1
debug = 0

# Log levels
ENTRY = '>> ENTRY: '
INFO = 'INFO: '
ERROR = 'ERROR: '
EXIT = '<< EXIT: '
WARNING = '!! WARNING: '

DEFAULT_TRACK_ACRONYM = "Main"

prefixStr = ''

def logMsg(s, prefix = True):
    '''Log the given string
'''
    if prefix:
        print prefixStr + " " + s
        print
    else:
        print s
        print    
#endDef

def undeploy(paAcronym, snapshotAcronym, trackAcronym, forTip = False):
    '''Undeploy a PA tip or snapshot
'''
    if forTip:
        logMsg(ENTRY + "Undeploying the tip of process application " + paAcronym + " for track " + trackAcronym)
        params = ["-containerAcronym", paAcronym, "-containerSnapshotAcronym", "Tip", "-containerTrackAcronym", trackAcronym]
    else:
        logMsg(ENTRY + "Undeploying snapshot " + snapshotAcronym + " of process application " + paAcronym + " for track " + trackAcronym)
        params = ["-containerAcronym", paAcronym, "-containerSnapshotAcronym", snapshotAcronym, "-containerTrackAcronym", trackAcronym]
    try:
        AdminTask.BPMUndeploy(params)
    except:        
        logMsg(ERROR + "Failed to undeploy the snapshot" + snapshotAcronym + " of process appliction " + paAcronym + " for track " + trackAcronym)
        logMsg(sys.exc_info()[1], False)
##        raise

        logMsg(EXIT + "Undeployed the snapshot " + snapshotAcronym + " of process application " + paAcronym + " for track " + trackAcronym)
#endDef

def archive(paAcronym, snapshotAcronym, trackAcronym):
    '''Archive a PA/Toolkit snapshot
'''

    logMsg(ENTRY + "Archive snapshot " + snapshotAcronym + " of container " + paAcronym + " ....")

    params = ["-containerAcronym", paAcronym, "-containerSnapshotAcronym", snapshotAcronym, "-containerTrackAcronym", trackAcronym]
    try:
        AdminTask.BPMArchive(params)
        ##   AdminConfig.save()  
    except:
        logMsg(ERROR + "Failed to archive snapshot " + snapshotAcronym + " of container " + paAcronym + " with error: ")
        logMsg(sys.exc_info()[1], False)
        ##   AdminConfig.reset()
        raise

    logMsg(EXIT + "Archive snapshot " + snapshotAcronym + " of container " + paAcronym) 

#endDef

def verifyState(paAcronym, trackAcronym, snapshotAcronym, expectedState):
    '''Verify state of the PA or snapshot (use "Tip" for snapshotAcronym for Tip)
'''
    state = getSnapshotState(paAcronym, snapshotAcronym, trackAcronym, False)
    if expectedState.strip() != state.strip():
        logMsg(WARNING + "Expected state " + expectedState + " of the snapshot does not match the actual state " + state)
        return False
    else:
        return True

#endDef

def getSnapshotState(paAcronym, snapshotAcronym, trackAcronym = "", debug = False):
    '''Get a snapshot state, given snapshot acronym and its containing process application
'''

    if debug:
        logMsg(ENTRY + "Get the state of snapshot " + snapshotAcronym + " of process application " + paAcronym + " ....")

    if trackAcronym == "":
        trackAcronym = DEFAULT_TRACK_ACRONYM
    params = ["-containerAcronym", paAcronym, "-containerSnapshotAcronym", snapshotAcronym, "-containerTrackAcronym", trackAcronym]

    state = "null";
    try:
        data = AdminTask.BPMShowSnapshot(params).splitlines()
        i = 1
        while i < len(data):
            if not data[i].strip().startswith("State: "):
                i = i + 1
                continue
            else:
                state = (data[i].strip().split("State: ")[1]).strip()
                break

    except:
        logMsg(ERROR + "Failed to get the state of snapshot " + snapshotAcronym + " of process application " + paAcronym + " with error: ")
        logMsg(sys.exc_info()[1], False)
        raise

    if debug:
            logMsg(EXIT + "Get the state of snapshot " + snapshotAcronym + " of container " + paAcronym)
    return state

#endDef
def deactivate(paAcronym, snapshotAcronym, trackAcronym = ""):
    '''Deactivate a PA/Toolkit snapshot
'''

    logMsg(ENTRY + "Deactivating snapshot " + snapshotAcronym + " of process application " + paAcronym + " for track " + trackAcronym)

    if trackAcronym == "":
        params = ["-containerAcronym", paAcronym, "-containerSnapshotAcronym", snapshotAcronym]
    else:
        params = ["-containerAcronym", paAcronym, "-containerSnapshotAcronym", snapshotAcronym, "-containerTrackAcronym", trackAcronym]

    try:
        AdminTask.BPMDeactivate(params)
    except:
        logMsg(ERROR + "Failed to deactivate snapshot " + snapshotAcronym + " of process application " + paAcronym + " with error: ")
        logMsg(sys.exc_info()[1], False)
        raise

    logMsg(EXIT + "Deactivated snapshot " + snapshotAcronym + " of process application " + paAcronym + " for track " + trackAcronym)

#endDef
def getListPA():
    '''Get a list of all PAs in the Process Center as string
'''
    if debug:
        logMsg(ENTRY + "Finding all process applications ")
    try:
        data = AdminTask.BPMListProcessApplications()
    except:
        logMsg(ERROR + "Failed to retrieve process applications with error: ")
        logMsg(sys.exc_info()[1], False)
        raise
    if debug:
        logMsg(EXIT + "finished getting PAs.")
    return data

#endDef

def parsePAList(data):
    '''Parse the string representing the list of PAs and 
return a list of all PAs. Each PA is represented by a dictionary.
'''
    if debug:
            logMsg(ENTRY + "Processing list of process applications")
    data = data.split('\n',1)[1]
    PAsStr = data.split('\n\nName: ')
    PAs = []
    for pa in PAsStr:
        pa = 'Name: ' + pa
        PA = fieldsTOdict(pa)
        PAs.append(PA)
    if debug:
            logMsg(EXIT + "Finished processing list of process applications")
    return PAs
#endDef

def fieldsTOdict(fieldStr):
    '''From a formatted string contaning FIELDs:VALUEs (1 per line) representing an 
object, parse the string and return the FIELDs and VALUEs as a dictionary.
suitable for PA descriptions, snapshot descriptions and others, as returned
by the wsadmin command. 
'''
    lines = fieldStr.splitlines()
    lines = filter(None, lines)
    d = {}
    for line in lines:
        line = line.strip()
        if not line:
            continue
        try:
            k,v = line.split(':', 1)
            d[k.strip()] = v.strip()
        except ValueError:
            d[line] = ''
        except:
            print repr(line)
            print 'lines:', lines
            raise
    return d

#endDef


def getAllSnapshots(paAcronym):
    '''Get details of all snapshots, given the acronym of the process application.
'''
    if debug:
        logMsg(ENTRY + "Retrieving snapshots of process application " + paAcronym + " ....")

    params = ["-containerAcronym", paAcronym]
    ssList = []
    try:
        data = AdminTask.BPMShowProcessApplication(params).strip()
        data = data.split('Tracks:',1)[1]
        tracks = data.split('Track Name:')
        for track in tracks:
            track = 'Track Name: ' + track
            trackInfo, snapshots = track.split('\n\n',1)
            trackDict = fieldsTOdict(trackInfo)
            # track info will be added to each snapshot
            snapshots = snapshots.replace('List of Snapshots:', '')
            snapshots = snapshots.replace('Tip:', 'Name: Tip \n Acronym: Tip')
            snapshots = snapshots.split('\n\n')
            snapshots = filter(None, snapshots)
            for ss in snapshots:
                s = fieldsTOdict(ss)
                if s:
                    s.update( trackDict )
                    s['PA'] = paAcronym
                    s['Days Old'] = howOld(s['Created On'])
                    ssList.append(s)
        pass
    except:
        if debug:
            logMsg(ERROR + "Failed to get the snapshots of process application " + paAcronym)
            logMsg(sys.exc_info()[1], False)
            print repr(track)
        raise
        pass
    if debug:
        logMsg(EXIT +  "Finished retrieving snapshots of process application " + paAcronym)
    return ssList

#endDef

def filter1(ss):
    '''track info will be added to each snapshot
Given the list of snapshots, filter and return a subset based on 
- Deactivated snapshots
- Have advanced content
- Created more than "AGE" days ago
- Have no running BPD instances
'''
    try:
        if  (
                ss['State'].count('Inactive')       and 
                howOld(ss['Created On']) >= AGE       and 
                ss['Capability'].count('Advanced')  and 
                ss['No of running instances'] == str(0)
            ):
                return True
        else:
                return False
    except KeyError:
        return False
    except TypeError:
        print repr(ss)
        raise

#endDef

def filter3(ss):
    '''Given the list of snapshots, filter and return a subset based on 
- Activated snapshots
- Have advanced content
- Created more than "AGE" days ago
- Have no running BPD instances
'''
    try:
        if  (
                ss['State'].count('Active')       and 
                howOld(ss['Created On']) >= AGE       and 
                ss['Capability'].count('Advanced')  and 
                ss['No of running instances'] == str(0)
            ):
                return True
        else:
                return False
    except KeyError:
        return False
    except TypeError:
        print repr(ss)
        raise

#endDef

def filter2(ss):
    '''Given the list of snapshots, filter and return a subset based on 
- Deactivated snapshots
- Have advanced content
- Created more than "AGE" days ago
- Have some running BPD instances
'''
    try:
        if  (
                ss['State'].count('Inactive')       and 
                howOld(ss['Created On']) >= AGE       and 
                ss['Capability'].count('Advanced')  and 
                int(ss['No of running instances']) > 0
            ):
                return True
        else:
                return False
    except KeyError:
        return False
    except TypeError:
        print repr(ss)
        raise

#endDef

def filter4(ss):
    '''Given the list of snapshots, filter and return a subset based on 
- Activated snapshots
- Have advanced content
- Created more than "AGE" days ago
- Have some running BPD instances
'''
    try:
        if  (
                ss['State'].count('Active')       and 
                howOld(ss['Created On']) >= AGE       and 
                ss['Capability'].count('Advanced')  and 
                int(ss['No of running instances']) > 0
            ):
                return True
        else:
                return False
    except KeyError:
        return False
    except TypeError:
        print repr(ss)
        raise

#endDef


def filterStd1(ss):
    '''Given the list of snapshots, filter and return a subset based on 
- Deactivated snapshots
- Have Standard content
- Created more than "AGE" days ago
- Have no running BPD instances
'''
    try:
        if  (
                ss['State'].count('Inactive')       and 
                howOld(ss['Created On']) >= AGE       and 
                ss['Capability'].count('Standard')  and 
                not ss['Acronym'].count('Tip')  and 
                ss['No of running instances'] == str(0)
            ):
                return True
        else:
                return False
    except KeyError:
        return False
    except TypeError:
        print repr(ss)
        raise

#endDef

def filterStd3(ss):
    '''Given the list of snapshots, filter and return a subset based on 
- Activated snapshots
- Have Standard content
- Created more than "AGE" days ago
- Have no running BPD instances
'''
    try:
        if  (
                ss['State'].count('Active')       and 
                howOld(ss['Created On']) >= AGE       and 
                ss['Capability'].count('Standard')  and 
                not ss['Acronym'].count('Tip')  and 
                ss['No of running instances'] == str(0)
            ):
                return True
        else:
                return False
    except KeyError:
        return False
    except TypeError:
        print repr(ss)
        raise

#endDef

def filterStd2(ss):
    '''Given the list of snapshots, filter and return a subset based on 
- Deactivated snapshots
- Have Standard content
- Created more than "AGE" days ago
- Have some running BPD instances
'''
    try:
        if  (
                ss['State'].count('Inactive')       and 
                howOld(ss['Created On']) >= AGE       and 
                ss['Capability'].count('Standard')  and 
                not ss['Acronym'].count('Tip')  and 
                int(ss['No of running instances']) > 0
            ):
                return True
        else:
                return False
    except KeyError:
        return False
    except TypeError:
        print repr(ss)
        raise

#endDef

def filterStd4(ss):
    '''Given the list of snapshots, filter and return a subset based on 
- Activated snapshots
- Have Standard content
- Created more than "AGE" days ago
- Have some running BPD instances
'''
    try:
        if  (
                ss['State'].count('Active')       and 
                howOld(ss['Created On']) >= AGE       and 
                ss['Capability'].count('Standard')  and 
                not ss['Acronym'].count('Tip')  and 
                int(ss['No of running instances']) > 0
            ):
                return True
        else:
                return False
    except KeyError:
        return False
    except TypeError:
        print repr(ss)
        raise

#endDef

def howOld(dateStr):
    '''Accepts a date string as: '2012-08-27 14:45:01.718'
and returns how many days have passed (rounded).
'''
    ctime = time.localtime()
    dateStr = dateStr.split()[0]
    dateList = dateStr.split('-')
    pasTime = [int(j) for j in dateList]
    ydiff = ctime[0] - pasTime[0]
    ddiff = ctime[2] - pasTime[2]
    totDays = ddiff + 365 * ydiff
    return totDays

#endDef

def printSS(ss):
    '''Prints some snapshot fields and values to standard output.

Only the fileds listed in plist are printed out.'''
    plist = ['PA',
             'Toolkit',
             'Track Acronym',
             'Name',
             'Acronym',
             'State',
             'Capability',
             'No of running instances',
             'Created On',
             'Days Old',
             ] #'Track Info', 
    for p in plist:
        print p+':', ss[p]#.replace('\n','|')
    print

#endDef

def undeploySSList(ssList):
    '''For a given list of snapshots ssList, undeploy the snapshots (works for Tips too).'''
    for ss in ssList:
        try:
            if ss['Acronym'] != 'Tip':
                if verifyState(ss['PA'], ss['Track Acronym'], ss['Acronym'], 'State[Active]'):
                        deactivate(ss['PA'], ss['Acronym'], ss['Track Acronym'])
            if ss['Capability'].count('Advanced'):
                undeploy(ss['PA'], ss['Acronym'], ss['Track Acronym'], ss['Acronym']=='Tip')
            if not DONT_ARCHIVE and ss['Acronym'] != 'Tip':
                if ss['Toolkit'] == 'true':
                    logMsg(WARNING+"This tool doesn't archive toolkits")
                else:
                    archive(ss['PA'], ss['Acronym'], ss['Track Acronym'])
        except:
            logMsg(ERROR + "Failed to undeploy snapshot: " + ss['Acronym'] + " of process application " + ss['PA'])

#endDef

def processUserInput(ssFiltered,activeInstances=False):
        '''For a given list of snapshots ssFiltered, display the snapshots and 
ask the user if it wants their undeployment. Only 'yes' triggers the action.
Extra warning will appear if activeInstances is True.'''
        for s in ssFiltered:
                printSS(s)

        if activeInstances:
                logMsg(WARNING + "Undeploying snapshot(s)/tip(s) which have running BPD instances may cause failure during process execution.")

        inp = raw_input('Would you like to undeploy/archive the above ' + str( len(ssFiltered) ) + ' snapshot(s)? (yes/no/skip) \n')
        print
        if inp == 'yes':
                undeploySSList(ssFiltered)

#endDef        

advFilters = [ 
                     # Filter: INACTIVE, ADVANCED, NO ACTIVE PROCESS INSTANCES
                    (filter1,  
                        'Finding INACTIVE snapshots having advanced capability and NO active process instances',
                        'Could not find any INACTIVE snapshot(s)/tip(s) having advanced capability and NO active process instances',
                    ) ,

                     # Filter: INACTIVE, ADVANCED, SOME ACTIVE PROCESS INSTANCES
                    (filter2,  
                        'Finding INACTIVE snapshots having advanced capability and SOME active process instances',
                        'Could not find any INACTIVE snapshot(s)/tip(s) having advanced capability and SOME active process instances',
                    ) ,

                     # Filter: ACTIVE, ADVANCED, NO ACTIVE PROCESS INSTANCES
                    (filter3,  
                        'Finding ACTIVE snapshots having advanced capability and NO active process instances',
                        'Could not find any ACTIVE snapshots having advanced capability and NO active process instances',
                    ) ,

                     # Filter: ACTIVE, ADVANCED, SOME ACTIVE PROCESS INSTANCES
                    (filter4,  
                        'Finding ACTIVE snapshots having advanced capability and SOME active process instances',
                        'Could not find any ACTIVE snapshots having advanced capability and SOME active process instances',
                    ) ,
            ]
stdFilters = [
                     # filterStd: INACTIVE, STANDARD, NO ACTIVE PROCESS INSTANCES
                    (filterStd1,
                        'Finding INACTIVE snapshots having standard capability and NO active process instances',
                        'Could not find any INACTIVE snapshot(s)/tip(s) having standard capability and NO active process instances',
                    ),

                     # filterStd: INACTIVE, STANDARD, SOME ACTIVE PROCESS INSTANCES
                    (filterStd2,
                        'Finding INACTIVE snapshots having standard capability and SOME active process instances',
                        'Could not find any INACTIVE snapshot(s)/tip(s) having standard capability and SOME active process instances',
                    ),

                     # filterStd: ACTIVE, STANDARD, NO ACTIVE PROCESS INSTANCES
                    (filterStd3,
                        'Finding ACTIVE snapshots having standard capability and NO active process instances',
                        'Could not find any ACTIVE snapshots having standard capability and NO active process instances',
                    ),

                     # filterStd: ACTIVE, STANDARD, SOME ACTIVE PROCESS INSTANCES
                    (filterStd4,
                        'Finding ACTIVE snapshots having standard capability and SOME active process instances',
                        'Could not find any ACTIVE snapshots having standard capability and SOME active process instances',
                    ),

              ]



def undeploySequence(ssList, filters = advFilters):
    '''Given a list of snapshots ssList, it will filter the list 
applying 1 filter at a time, and asking the user for their undeployment.
'''
    if DONT_ARCHIVE:
        pass
    else:
        filters.extend(stdFilters)

    for currFilter, entryMessage, exitMessage in filters:
        logMsg(ENTRY + currFilter.__name__+': '+ entryMessage)

        ssFiltered = filter(currFilter, ssList)
        
        if not ssFiltered:
            logMsg(EXIT + currFilter.__name__+': '+ exitMessage)
        else:
            ieSS = ssFiltered[0]
            activeInstances = int( ieSS['No of running instances'] )
            processUserInput(ssFiltered,activeInstances)

#endDef    

systemPAs = [
            "SYSCM" , 
            "TWSYSG"  , 
            "SYSD"  ,
            "SYSC"  ,
            "TWSYS"  ,
            "SSA"  , 
            "TWP"  , 
            ]

def isNotSystemPAs(pa):
    return not pa['Acronym'] in systemPAs

#endDef    


def main():
    '''The main flow of the script.'''
    data = getListPA()
    PAs = parsePAList(data)
    PAs = filter(None, PAs)  # remove empty PAs
    PAs = filter(isNotSystemPAs, PAs)  # remove system PAs
    # if the user used the -pa_acronym flag, process only that Process Application
    if PA_TARGET:
        for pa in PAs:
            if pa['Acronym'] == PA_TARGET:
                PAs = [pa]
                break
        else:
            if PA_TARGET not in systemPAs:
                logMsg(WARNING +'PA acronym ' + PA_TARGET + ' could not be found. Make sure the spelling is correct.')
            else: 
                logMsg(WARNING + 'The acronym ' + PA_TARGET + ' is a system application and cannot be modified by this script.')
            logMsg(INFO + 'Continuiing with default beharvior.')
    snapshots = []
    for pa in PAs:
        try:
            # get snapshot for current PA and and Toolkit information
            ssList = getAllSnapshots(pa['Acronym'])
            for ss in ssList:
                ss['Toolkit'] = pa['Toolkit']
                if debug:
                    # just for debugging: print out snapshots with a different state than these below
                    if not ss['State'] in ['State[Active]', 'State[Inactive]', 'State[Undeployed]', 'State[Archived]']:
                        printSS(ss)
            snapshots.extend( ssList )
        except KeyError:
            print repr(pa)
            print repr(ssList)
            raise  # PAs with newlines in descriptions might pass with this line removed
    if PA_TARGET and snapshots and snapshots[0]['Capability'] == 'Capability[Standard]' and DONT_ARCHIVE:
        logMsg(WARNING + 'For applications with standard content use the -archive flag.')
    else:
        undeploySequence(snapshots)

#endDef    

def manageArgv():
    '''Takes care of the commandline arguments.
    Needed because this version of Jython doesn't have the optparse/argvparse library.'''
    global AGE, DONT_ARCHIVE, PA_TARGET
    argv = sys.argv
    if argv:
        if '-age' in argv:
            try:
                AGE = int(argv[argv.index('-age')+1])
            except:
                logMsg(WARNING+'Could not understand the -age flag in these commandline options: ' + str(argv))
                logMsg(WARNING+'Continuing with the age default value of '+str(AGE))
        if '-archive' in argv:
            DONT_ARCHIVE = False
        if '-paAcronym' in argv:
            try:
                PA_TARGET = argv[argv.index('-paAcronym')+1]
            except:
                logMsg(WARNING+'Could not understand the -paAcronym flag in these commandline options: ' + str(argv))
                logMsg(WARNING+'Continuing with the default behavior')




if __name__ == "__main__":
    # start cleanup
    try:
        logMsg(INFO + 'Script started.')
        manageArgv()
        main()
    finally:
        logMsg(INFO + 'Script ended.')









