/*
Licensed Materials - Property of IBM
5725-C94, 5725-C95, 5725-C96, 5655-Y02
© Copyright IBM Corporation 2012. All Rights Reserved.
This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer 
(a) for its own instruction and study, (b) in order to develop applications designed to run with an IBM product, 
either for customer's own internal use or for redistribution by customer, as part of such an application, in customer's 
own products.
*/

package com.ibm.bpm.integration.filenet;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;

public class ECMEventPoster {
        
    private static String CLASS = ECMEventPoster.class.getName();
    private Logger logger = Logger.getLogger(CLASS);
    
    BPMClientConfig config = null;
    AuthScope authScope = null;
    UsernamePasswordCredentials creds;
    
    
    
    public ECMEventPoster(BPMClientConfig config) {
        if (logger.isLoggable(Level.FINEST)) logger.entering(CLASS, "<init>(BPMClientConfig) ", config);
        
        this.config = config;
        this.authScope = new AuthScope(config.getTargetServer().getHost(), config.getTargetServer().getPort());
        this.creds = new UsernamePasswordCredentials(config.getUserName(), config.getPassword());
        
        if (logger.isLoggable(Level.FINEST)) logger.exiting(CLASS, "<init>(BPMClientConfig) ");
    }
    
    
    public int postEvent(ECMEvent event) throws HttpException, IOException {
        String methodName = "postEvent";
        if (logger.isLoggable(Level.FINEST)) logger.entering(CLASS, methodName, event);
        
        int statusCode = 500;
        
        HttpClient httpClient = new  HttpClient();
        httpClient.getState().setCredentials(authScope, creds);
        
        statusCode = httpClient.executeMethod(getHttpMethod(event));

        if (logger.isLoggable(Level.FINEST)) logger.exiting(CLASS, methodName, statusCode);
        return statusCode;
        
    }

    private HttpMethod getHttpMethod(ECMEvent event) {  
        HttpMethod method = new PostMethod(getPostEventURL(event));
        method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, new DefaultHttpMethodRetryHandler(0, false));
        return method;
    }


    private String getPostEventURL(ECMEvent event) {
        String methodName = "getPostEventURL";
        if (logger.isLoggable(Level.FINEST)) logger.entering(CLASS, methodName, event);
        
        StringBuffer buf = new StringBuffer(config.getTargetServer().toString());
        buf.append("/rest/bpm/wle/v1/event/ecm?eventSourceId=").append(event.getEventSourceId());
        buf.append("&objectTypeId=").append(event.getObjectTypeId());
        buf.append("&eventType=").append(event.getEventType());
        buf.append("&objectId=").append(event.getObjectId());
        
        if (logger.isLoggable(Level.FINEST)) logger.exiting(CLASS, methodName, buf.toString());
        return buf.toString();
    }

    
    
}
